#ifndef ASTROTIME_H_LOADED
#define ASTROTIME_H_LOADED 1

#include <stdlib.h>

#if PROVARINO
#include <string>
#endif

// Ratio of sidereal time to UT, as of 2020 (0.2 Julian century from 2000)
// thus 1.002737909350795 + 5.9006e-11 * 0.2
#define STperUT 1.0027379093625963247

/**
 * Encapsulates a few time properties which are astronomically relevant.
 */
class AstroTime {
 private:
    double ut24;          // UT, as a number of hours
    double obslong;       // east-longitude of observatory, in radians
    int mjd_ut0;          // MJD of 0h UT1 on specified date

    mutable double gmst; // Greenwich Mean Sidereal Time of date, in radians

 public:
    const double gmst_2pi(void) const;
    const double lmst(void) const;
    const double ut1(void) const;
    const int mjd(void) const;

#if PROVARINO
    std::string ut1_str(void) const;
    std::string ymd_str(void) const;
    std::string str(void) const;
#endif

    AstroTime();
    AstroTime(const int year,
              const int month,
              const int day,
              const int hour,
              const int minute,
              const double second,
              const double obslong);
    // FIXME: This would be tidier, perhaps, as method
    // delayed(extra_secs) (or later(), or inc_time()), by symmetry
    // with move(obs_lon)
    AstroTime(const AstroTime& ref, const double extra_secs);
    AstroTime(const int year,
              const int month,
              const int day);
    //AstroTime(const struct tm*, const double obslong);

    AstroTime move(double obs_lon);

    char* str(char*, size_t);
};

#if PROVARINO
std::ostream& operator<<(std::ostream &strm, const AstroTime& a);
#endif

#endif /* ASTROTIME_H_LOADED */
