#include "exception.h"
#include "qp.h"

#include "buffered-serial.h"
extern BufferedSerial bs;


/*
 * The only 'exception' handling is a function qp_error(is_fatal, format, ...).
 * This does different things in the test code (where it actually
 * throws a string) and the real Arduino environment (where it calls
 * a special function in qp.ino).
 */

#include <stdarg.h>

static void qp_error_(bool is_fatal, const char* file, unsigned int line, PGM_P format, va_list ap);

void qp_error(bool is_fatal, const char* file, unsigned int line, PGM_P format, ...)
{
    va_list ap;
    va_start(ap, format);
    qp_error_(is_fatal, file, line, format, ap);
    va_end(ap);
}

#if PROVARINO
#define MSGLEN 128
#include <string.h>

// For test code
void qp_error_(bool is_fatal, const char* file, unsigned int line, PGM_P format, va_list ap)
{
    static char msg_buf[MSGLEN];
    char* p = msg_buf;
    size_t pfxlen = snprintf(msg_buf, MSGLEN, "%s (%s:%d): ",
                             (is_fatal ? "fatal: " : "recoverable: "),
                             file, line);
    p = &msg_buf[pfxlen];
    vsnprintf(p, MSGLEN-pfxlen, format, ap);
    throw msg_buf;
}

#else

// For the arduino environment

#include "util.h"

void qp_error_(bool is_fatal, const char* file, unsigned int line, PGM_P format, va_list ap)
{
    bs.logprintf(PSTR("!%s (%s:%d): %|"), (is_fatal ? "fatal" : "error"), file, line);
    bs.logvprintf(format, ap);

    // now call the fault_exception function in the main program,
    // which longjumps into the fault-handling code
    fault_exception(is_fatal);
}
#endif
