#include "rotation.h"

#include <math.h>

Rotation::Rotation(const Quaternion start, const Quaternion axis)
    : A(start), u(axis),
      Au(Quaternion(Quaternion::Unit)), uxA(Quaternion(Quaternion::Unit)) // sentinel values
{
    // empty
}

double Rotation::intersection(const Quaternion& b, const Quaternion& P) const
{
    // See notes `quaternion-pointing`, section `meridians`.
    if (Au.is_unit_p()) {
        Au = A.reflect(u);
        uxA = u.cross(A);
    }

    double t0 = b.dot(A + A.reflect(b));
    double t1 = b.dot(uxA + uxA.reflect(b));
    double t2 = b.dot(Au + Au.reflect(b));

    double result = 0.0;

    double t1t0t2 = t1*t1 - t0*t2;
    if (t1t0t2 >= 0) {
        t1t0t2 = sqrt(t1t0t2);
        double tan_theta[2] = {(-t1 + t1t0t2)/t2,
                               (-t1 - t1t0t2)/t2 };

        for (int i=0; i<2; i++) {
            double tan2 = tan_theta[i]*tan_theta[i];
            double cos2 = 1/(1+tan2);
            double cot2 = 1/tan2;
            double sin2 = 1/(1+cot2);
            double sc2 = sqrt(2 + tan2 + cot2)/4;
            if (tan_theta[i] < 0) sc2 = -sc2;

            Quaternion B = A.scale(cos2) + Au.scale(sin2) + uxA.scale(sc2);
            double ip = b.dot(P.cross(B));

            //COUT << "B=" << B << "; ip=" << ip << endl;

            if (ip > 0) {
                if (result == 0.0 || tan_theta[i] < result) {
                    result = tan_theta[i];
                }
            }
        }
    } else {
        // this rotation does not cross the great circle at all
        result = 0.0;
    }

    //COUT << "result=" << result << endl;
    return result;
}

double Rotation::intersection(const Quaternion& b, const float h) const
{
    // See notes `quaternion-pointing`, section `meridians`.
    if (Au.is_unit_p()) {
        Au = A.reflect(u);
        uxA = u.cross(A);
    }

    double t0 = b.dot(A + A.reflect(b)) - 2*h;
    double t1 = b.dot(uxA + uxA.reflect(b));
    double t2 = b.dot(Au + Au.reflect(b)) - 2*h;

    double result;

    double t1t0t2 = t1*t1 - t0*t2;
    if (t1t0t2 >= 0) {
        t1t0t2 = sqrt(t1t0t2);

        double tan_theta1 = (-t1 + t1t0t2)/t2;
        double tan_theta2 = (-t1 - t1t0t2)/t2;

        // return the tan_theta which corresponds to the smaller angle
        // between 0 and pi.
        if (tan_theta1 * tan_theta2 >= 0) {
            // same sign -- return smaller
            result = tan_theta1 < tan_theta2 ? tan_theta1 : tan_theta2;
        } else {
            // opposite sign -- return the positive one
            result = tan_theta1 >= 0 ? tan_theta1 : tan_theta2;
        }
    } else {
        // this rotation does not cross the great circle at all
        result = 0.0;
    }

    //COUT << "result=" << result << endl;
    return result;
}
